// -----------------------------------------
// EasyExpert Order Functions - Include File 
// by Andrew Young - andrew@inmyroom.org
// Version 1.0 beta - Dec 2007
// -----------------------------------------


// Place this file in your /experts/include folder.
 
// It is not necessary to compile this file, nor is 
// it required for your compiled expert advisor to run.

// To use these functions in your expert advisor, 
// add this line to the beginning of your .mq4 file:
//
//    #include <EasyExpert.mqh>
//
// See the provided EasyExpertMACross.mq4 file
// for an example on how to use these functions.


// External Variables

  extern string Settings = "**** Expert Settings ****";
  extern int InstanceID = 123;          // "Magic number" - Allows seperate instances of EA to run
  extern int MinMargin = 100;           // Minimum margin required to open an order
  extern int Slippage = 5;

  extern string MoneyManagement = "**** Money Management Settings ****";
  extern bool UseMoneyManagement = true;
  extern double MinRisk = 2;            // Starting order size as % of equity
  extern double MaxRisk = 6;            // Maximum % of equity in all open orders 
  extern int MaxOrders = 6;             // Maximum # of open orders 
  extern double LotSize = 0.1;          // Static lot size - use when UseMoneyManagement = false
  
  extern string TradeSettings = "**** Initial Trade Settings ****";
  extern int StopLoss;
  extern int TrailingStop;
  extern int TakeProfit;
  
  extern string AddProfitable = "**** Add to Profitable Trades ****";
  extern bool AddToProfit = true;
  extern double AddMultiplier = 0.5;    // Multiplier for add orders
  extern int AddTrigger;                // Open new order for every X pips of profit
  extern int AddStop; 
  extern int AddTrailing;    
  extern int AddTakeProfit;
  
  extern string Hedge = "**** Hedge Unprofitable Trades ****";
  extern bool Hedging = true;
  extern double HedgeMultiplier = 2;    // Multiplier for hedge orders
  extern int HedgeTrigger;              // Open new order on X pips of loss
  extern int HedgeStop;
  extern int HedgeTrailing;
  extern int HedgeTakeProfit;
  

// Static variables
  
  
  static bool BuyOpen = false;          // Buy order currently open
  static bool SellOpen = false;         // Sell order currently open  
  static int BuyAddCount = 0;           // Buy add orders open
  static int SellAddCount = 0;          // Sell add orders open
  static bool SellHedgeOpen = false;    // Sell hedge (buy) order open 
  static bool BuyHedgeOpen = false;     // Buy hedge (sell) order open 
  static datetime CurrentBar;


// ---------------------------------------------
// External Functions - Call these from your EA
// --------------------------------------------- 


// Include at the beginning of your start() function
// if you want to run your expert only once per bar.

int OncePerBar()
  {
    if (CurrentBar == iTime(NULL,0,0)) return(0);
    CurrentBar = iTime(NULL,0,0);
    return(0);
  }


// Order status checks. Add to the beginning of your start() function
// Required for proper operation

int OrderCheck()
  {
  int i;
  bool BuyFound;
  bool SellFound;
  bool BuyHedgeFound;
  bool SellHedgeFound;
  BuyAddCount = 0;
  SellAddCount = 0;
  for (i=OrdersTotal()-1; i>=0; i--)
    {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
      if (OrderType() == OP_BUY && OrderMagicNumber() == InstanceID && OrderSymbol() == Symbol())
        {
          if (StringFind(OrderComment(),"Initial Buy",0) != -1) BuyFound = true;
          if (StringFind(OrderComment(),"Added",0) != -1) BuyAddCount++;
          if (StringFind(OrderComment(),"Hedging Sell",0) != -1) SellHedgeFound = true;
        }
      if (OrderType() == OP_SELL && OrderMagicNumber() == InstanceID && OrderSymbol() == Symbol())
        {
          if (StringFind(OrderComment(),"Initial Sell",0) != -1) SellFound = true;
          if (StringFind(OrderComment(),"Added",0) != -1) SellAddCount++;
          if (StringFind(OrderComment(),"Hedging Buy",0) != -1) BuyHedgeFound = true;
        }
    }
      if(BuyFound == false && BuyOpen == true) BuyOpen = false; 
      if(SellFound == false && SellOpen == true) SellOpen = false;
      if(BuyHedgeFound == false && BuyHedgeOpen == true) BuyHedgeOpen = false; 
      if(SellHedgeFound == false && SellHedgeOpen == true) SellHedgeOpen = false;
  return(0);
  }


// Open an initial sell order

int OpenSellOrder()
  {
  double Lots = CalcLotSize();
  if (TotalOrders() < MaxOrders && SellOpen == false && AccountFreeMarginCheck(Symbol(),OP_SELL,Lots) > MinMargin)
    {
      OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,Bid+(StopLoss*Point),Bid-(TakeProfit*Point),"Initial Sell Order",InstanceID,0,ForestGreen);
      SellOpen = true;
      Comment("Sell order opened");
    }
  return(0);
  }


// Open an initial buy order

int OpenBuyOrder()
  {
  double Lots = CalcLotSize();
  if (TotalOrders() < MaxOrders && BuyOpen == false && AccountFreeMarginCheck(Symbol(),OP_BUY,Lots) > MinMargin)
    {
      OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,Ask-(StopLoss*Point),Ask+(TakeProfit*Point),"Initial Buy Order",InstanceID,0,ForestGreen);
      BuyOpen = true;
      Comment("Buy order opened");
    }
  return(0);
  }


// Close all buy orders

int CloseBuyOrders()
  {
  int i;
  for (i=OrdersTotal()-1; i>=0; i--)
    {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
      RefreshRates();
      if (OrderType() == OP_BUY && OrderMagicNumber() == InstanceID && OrderSymbol() == Symbol())
        {
          OrderClose(OrderTicket(),OrderLots(),Ask,Slippage,Crimson);
          BuyOpen = false;
          BuyAddCount = 0;
        }
    }
  return(0);
}


// Close all sell orders

int CloseSellOrders()
  {
  int i;
  for (i=OrdersTotal()-1; i>=0; i--)
    {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
      RefreshRates();
      if (OrderType() == OP_SELL && OrderMagicNumber() == InstanceID && OrderSymbol() == Symbol())
        {
          OrderClose(OrderTicket(),OrderLots(),Bid,Slippage,Crimson);
          SellOpen = false;
          SellAddCount = 0;
        }
    }
  return(0);
}


// Add new order to profitable sell position

int AddToSellOrder()
  {
  if(AddToProfit == true && (SellOpen == true || BuyHedgeOpen == true))
    {
      double AddLots = CalcAddLotSize();
      double ModAddLots = RiskAmount(AddLots);
      if (ModAddLots == 0) return(0);
      double LotTrigger = AddTrigger*(SellAddCount+1)*Point;
      int OpenOrders = TotalOrders();
      int i;
      for (i=OrdersTotal()-1; i>=0; i--)
        {
          OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
          RefreshRates();
          double SellProfit = OrderOpenPrice()-Bid;
          if(OrderType() == OP_SELL && OrderMagicNumber() == InstanceID && OrderSymbol() == Symbol() && SellProfit > LotTrigger 
          && OpenOrders < MaxOrders && AccountFreeMarginCheck(Symbol(),OP_SELL,ModAddLots) > MinMargin && StringFind(OrderComment(),"Added",0) == -1) 
            {
              string AddTicket = OrderTicket();
              OrderSend(Symbol(),OP_SELL,ModAddLots,Bid,Slippage,Bid+(AddStop*Point),Bid-AddTakeProfit*Point,"Added to "+AddTicket,InstanceID,0,DarkViolet);
              Comment("Added to profitable sell position");
              if (SellAddCount+1 < MaxOrders) SellAddCount++;
            }
        }
    }
  return(0);
}


// Add new order to profitable buy position

int AddToBuyOrder()
  {
  if(AddToProfit == true && (BuyOpen == true || SellHedgeOpen == true))
    {
      double AddLots = CalcAddLotSize();
      double ModAddLots = RiskAmount(AddLots);
      if (ModAddLots == 0) return(0);
      double LotTrigger = AddTrigger*(BuyAddCount+1)*Point;
      int OpenOrders = TotalOrders();
      int i;
      for (i=OrdersTotal()-1; i>=0; i--)
        {
          OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
          RefreshRates();
          double BuyProfit = Ask-OrderOpenPrice();
          if(OrderType() == OP_BUY && OrderMagicNumber() == InstanceID && OrderSymbol() == Symbol() && BuyProfit > LotTrigger 
          && OpenOrders < MaxOrders && AccountFreeMarginCheck(Symbol(),OP_BUY,ModAddLots) > MinMargin && StringFind(OrderComment(),"Added",0) == -1)  
            {
              string AddTicket = OrderTicket();
              OrderSend(Symbol(),OP_BUY,ModAddLots,Ask,Slippage,Ask-(AddStop*Point),Ask+AddTakeProfit*Point,"Added to "+AddTicket,InstanceID,0,DarkViolet);
              Comment("Added to profitable buy position");
              if (BuyAddCount+1 < MaxOrders) BuyAddCount++;
            }
        }
    }
  return(0);
}


// Open buy order against losing sell position

int OpenSellHedge()
{
  if(Hedging == true && (SellOpen == true || SellAddCount > 0))
    {
      int OpenOrders = TotalOrders();
      int i;
      for (i=OrdersTotal()-1; i>=0; i--)
        {
          OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
          RefreshRates();
          if (OrderType() == OP_SELL && OrderMagicNumber() == InstanceID && OrderSymbol() == Symbol() && -HedgeTrigger*Point > OrderOpenPrice()-Bid 
          && OpenOrders < MaxOrders && SellHedgeOpen == false && StringFind(OrderComment(),"Hedging",0) == -1)
            {
              string SellHedgeTicket = OrderTicket();
              double HedgeLots = OrderLots() * HedgeMultiplier;
              double ModHedgeLots = RiskAmount(HedgeLots);
              if (ModHedgeLots == 0) return(0);
              bool OpenHedge = true;
            }
        }
         if (OpenHedge == true)
          {
            if (AccountFreeMarginCheck(Symbol(),OP_BUY,ModHedgeLots) > MinMargin)
              {
                OrderSend(Symbol(),OP_BUY,ModHedgeLots,Ask,Slippage,Ask-(HedgeStop*Point),Ask+HedgeTakeProfit*Point,"Hedging Sell Order "+SellHedgeTicket,InstanceID,0,Yellow);
                Comment("Sell hedge opened");
                SellHedgeOpen = true;
              }
          }
        
    }    
  return(0);
}


// Open sell order against losing buy position

int OpenBuyHedge()
{
  if(Hedging == true && (BuyOpen == true ||BuyAddCount > 0))
    {
      int OpenOrders = TotalOrders();
      int i;
      for (i=OrdersTotal()-1; i>=0; i--)
        {
          OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
          RefreshRates();
          if (OrderType() == OP_BUY && OrderMagicNumber() == InstanceID && OrderSymbol() == Symbol() && -HedgeTrigger*Point > Ask-OrderOpenPrice() 
          && OpenOrders < MaxOrders && BuyHedgeOpen == false && StringFind(OrderComment(),"Hedging",0) == -1)
            {
              string BuyHedgeTicket = OrderTicket();
              double HedgeLots = OrderLots() * HedgeMultiplier;
              double ModHedgeLots = RiskAmount(HedgeLots);
              if (ModHedgeLots == 0) return(0);
              bool OpenHedge = true;
            }
        }
          if (OpenHedge == true)
            {
              if (AccountFreeMarginCheck(Symbol(),OP_BUY,ModHedgeLots) > MinMargin)
                {
                  OrderSend(Symbol(),OP_SELL,ModHedgeLots,Bid,Slippage,Bid+(HedgeStop*Point),Bid-HedgeTakeProfit*Point,"Hedging Buy Order "+BuyHedgeTicket,InstanceID,0,Yellow);
                  Comment("Buy hedge opened");
                  BuyHedgeOpen = true;
                }
            }
        
    }    
  return(0);
}


// Trailing stops. Place this at the end of your start() function.

int AdjustTrailingStops()
{
  int i;
  for (i=OrdersTotal()-1; i>=0; i--)
    {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
      RefreshRates();
      if (OrderType() == OP_SELL && OrderMagicNumber() == InstanceID && OrderSymbol() == Symbol())
        {
          //Trailing stop
          if (StringFind(OrderComment(),"Initial Sell",0) != -1)
            {
              if (Bid+TrailingStop*Point < OrderOpenPrice() && Bid+TrailingStop*Point < OrderStopLoss())
                {
                  OrderModify(OrderTicket(),OrderOpenPrice(),Bid+TrailingStop*Point,OrderTakeProfit(),0);
                }  
            }
          //Add trailing stop
          if (StringFind(OrderComment(),"Added to",0) != -1)
            {
              if (Bid+AddTrailing*Point < OrderOpenPrice() && Bid+AddTrailing*Point < OrderStopLoss())
                {
                  OrderModify(OrderTicket(),OrderOpenPrice(),Bid+AddTrailing*Point,OrderTakeProfit(),0);
                }  
            }
          //Hedge trailing stop
          if (StringFind(OrderComment(),"Hedging Buy",0) != -1)
            {
              if (Bid+HedgeTrailing*Point < OrderOpenPrice() && Bid+HedgeTrailing*Point < OrderStopLoss())
                {
                  OrderModify(OrderTicket(),OrderOpenPrice(),Bid+HedgeTrailing*Point,OrderTakeProfit(),0);
                }  
            }
        }
      if (OrderType() == OP_BUY && OrderMagicNumber() == InstanceID && OrderSymbol() == Symbol())
        {
          //Trailing stop
          if (StringFind(OrderComment(),"Initial Buy",0) != -1)
            {
              if (Ask-TrailingStop*Point > OrderOpenPrice() && Ask-TrailingStop*Point > OrderStopLoss())
                {
                  OrderModify(OrderTicket(),OrderOpenPrice(),Ask-TrailingStop*Point,OrderTakeProfit(),0);
                }  
            }
          //Add trailing
          if (StringFind(OrderComment(),"Added to",0) != -1)
            {
              if (Ask-AddTrailing*Point > OrderOpenPrice() && Ask-AddTrailing*Point > OrderStopLoss())
                {
                  OrderModify(OrderTicket(),OrderOpenPrice(),Ask-AddTrailing*Point,OrderTakeProfit(),0);
                }  
            }
          //Hedge trailing
          if (StringFind(OrderComment(),"Hedging Sell",0) != -1)
            {
              if (Ask-HedgeTrailing*Point > OrderOpenPrice() && Ask-HedgeTrailing*Point > OrderStopLoss())
                {
                  OrderModify(OrderTicket(),OrderOpenPrice(),Ask-HedgeTrailing*Point,OrderTakeProfit(),0);
                }  
            } 
        }
    }
  return(0);
}


// --------------------
//  Internal functions
// --------------------

double CalcLotSize()
  {  
  if(UseMoneyManagement == true)
    {
      double SingleLot = MarketInfo(Symbol(),15)/AccountLeverage();
      double LotValue = AccountEquity()* MinRisk/100;
      if(MarketInfo(Symbol(),23) == 0.1)
        {
          double Lots = NormalizeDouble(LotValue/SingleLot,1);
          if (Lots < 0.1) Lots = 0.1;
        }
      else Lots = NormalizeDouble(LotValue/SingleLot,2);
    }
  else Lots = LotSize;
  if(Lots > MarketInfo(Symbol(),25)) Lots = MarketInfo(Symbol(),25);
  return(Lots);
  }

double CalcAddLotSize()
  {  
    double Lots = CalcLotSize();
    double AddLots = AddMultiplier * Lots;
    if(MarketInfo(Symbol(),23) == 0.1) AddLots = NormalizeDouble(AddLots,1);
    else AddLots = NormalizeDouble(AddLots,2);
    if (AddLots > MarketInfo(Symbol(),25)) AddLots = MarketInfo(Symbol(),25);
    return(AddLots);
  }

double RiskAmount(double RiskLot)
  {
  int i;
  double AllLots;
  double TotalRisk;
  double LotRisk;
  double ModRisk;
  double LotValue;
  double Lots;
  double SingleLot = MarketInfo(Symbol(),15)/AccountLeverage();
  
  for (i=OrdersTotal()-1; i>=0; i--)
    {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
      AllLots += OrderLots();
    }
  TotalRisk = (AllLots*SingleLot)/AccountEquity();
  if(TotalRisk >= MaxRisk/100) return(0);

  LotRisk = (RiskLot*SingleLot)/AccountEquity();
  if(TotalRisk+LotRisk <= MaxRisk/100) return(RiskLot);
  
  ModRisk = MaxRisk/100 - TotalRisk;
  LotValue = AccountEquity()* ModRisk;
  Lots = LotValue/SingleLot;
  if(MarketInfo(Symbol(),23) == 0.1)
    {
      Lots = NormalizeDouble(Lots,1);
    }
  else Lots = NormalizeDouble(Lots,2);
  return(Lots);
  } 
  
int TotalOrders()
  {
  int Orders = 0;
  int i;
  for (i=OrdersTotal()-1; i>=0; i--)
    {
    OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
    if (OrderMagicNumber() == InstanceID) Orders++;
    }
  return(Orders);
  }